#ifndef HAL_MGR_H_
#define HAL_MGR_H_

/* The hal manager connects to slave and master sides BUT a runtime side
 * connects to the hal manager by its own.
 *
 * Basiclly the runtime initializes reading/writing operations from/to slave
 * and master sides.
 */

/* Client sides. When we add a new client we use it as the `side'
 * argument.
 */
enum {
	HAL_CLT_SLAVE,
	HAL_CLT_MASTER,
	HAL_CLT_RUNTIME
};

/* Init internals manager structures. */
int hal_mgr_init(struct io_context *io_ptr);

/* Deinit manager and free resources of all clients. */
void hal_mgr_deinit();

/* Add a new client, rfd and wfd must be valid. */
int hal_mgr_client_raw_new(const char *name, int wfd, int rfd,
			   int timeout, int side);

/* Add a new client, we'll connect to a local socket. */
int hal_mgr_client_unix_new(const char *name, const char *sock,
			    int timeout, int side);

/* Add a new client, we'll connect to a specified inet address.
 * inet_addr looke like hostname:port or ipaddr:port.
 */
int hal_mgr_client_inet_new(const char *name, const char *inet_addr,
			    int timeout, int side);

/* Send a connect request command, to all master and slave sides.
 * The function return -1 even if only one client not connected.
 */
int hal_mgr_clients_connect();

/* If there are some error state clients, reconnect to them. */
void hal_mgr_clients_reconnect();

#endif

